package view;

import javafx.scene.paint.Color;
import javafx.scene.shape.Polygon;
import model.Move;
import model.Tile;

/**
 * Class that makes the association between a tile from the model and its representation
 */
public class TileView
{
	private Polygon fxTile;
	private Tile modelTile;
	private boolean selected, highlighted;
	private Move highlightMove = null; // How do we get here from the selected tile (used when highlighting)

	public TileView(Polygon fxTile, Tile modelTile)
	{

		this.fxTile = fxTile;
		this.modelTile = modelTile;
	}

	/**
	 * Tell the view to update according to the model
	 */
	public void update()
	{
		//TODO: Better tile representation
		Color fillColor = null;
		//FISH NUMBER
		switch (modelTile.getNbFish())
		{
			case 0:
				fillColor = Color.LIGHTBLUE;
				break;
			case 1:
				fillColor = Color.GREENYELLOW;
				break;
			case 2:
				fillColor = Color.GREEN;
				break;
			case 3:
				fillColor = Color.DARKGREEN;
				break;
		}
		//PENGUIN
		if (modelTile.getPenguinPresence().equals(Tile.PenguinPresence.RED_PENGUIN))
			fillColor = Color.RED;
		else if (modelTile.getPenguinPresence().equals(Tile.PenguinPresence.BLUE_PENGUIN))
			fillColor = Color.BLUE;

		//SELECTION/HIGHLIGHT
		if (selected)
			fillColor = fillColor.deriveColor(0, 1, 0.5, 1);
		if (highlighted)
			fillColor = fillColor.deriveColor(0, 0.5, 1, 1);

		fxTile.setFill(fillColor);

	}

	public Tile getModelTile() { return modelTile; }

	public boolean isSelected() { return selected; }

	public void setSelected(boolean selected) { this.selected = selected; }

	public boolean isHighlighted() { return highlighted; }

	public void setHighlighted(boolean highlighted) { this.highlighted = highlighted; }

	public Move getHighlightMove() { return highlightMove; }

	public void setHighlightMove(Move highlightMove) { this.highlightMove = highlightMove; }

}
